
package cz.cuni.mff.corrector.cache;

import cz.cuni.mff.corrector.intent.Intent;
import java.util.Collection;
import java.util.EnumMap;
import java.util.HashMap;
import java.util.Map;

/**
 * Flat implementation of the store manager based on signature indexing.
 *
 * @param <Item> Type of object to be stored by the cache manager instance.
 * @author Martin Svoboda
 */
public class SignatureStoreFlat<Item> implements Store<Item> {

    // -------------------------------------------------------------------------


    /**
     * Internal storage for all indexing intent types.
     * Indexing: type, signature
     */
    private Map<Intent.Type, Map<String, Item>> itemStorage;


    // -------------------------------------------------------------------------


    /**
     * Creates a new and empty caching manager with initialized storage.
     */
    public SignatureStoreFlat() {
        this.itemStorage =
            new EnumMap<Intent.Type, Map<String, Item>>(Intent.Type.class);
        for (Intent.Type type : Intent.Type.values()) {
            this.itemStorage.put(type, new HashMap<String, Item>());
        }
    }


    // -------------------------------------------------------------------------


    public Item getStoredItem(Intent indexingIntent) {
        return this.itemStorage.get(indexingIntent.getIntentType()).
            get(indexingIntent.getGlobalSignature());
    }

    
    public void setStoredItem(Intent indexingIntent, Item itemRef) {
        this.itemStorage.get(indexingIntent.getIntentType()).
            put(indexingIntent.getGlobalSignature(), itemRef);
    }


    public void removeStoredItem(Intent indexingIntent) {
        this.itemStorage.get(indexingIntent.getIntentType()).
            remove(indexingIntent.getGlobalSignature());
    }


    public Collection<Item> getStoredItems(Intent.Type intentType) {
        return this.itemStorage.get(intentType).values();
    }


    // -------------------------------------------------------------------------

}
