
package cz.cuni.mff.corrector.edit;

import cz.cuni.mff.corrector.instruction.Instruction;
import cz.cuni.mff.corrector.instruction.InstructionAddNode;
import cz.cuni.mff.corrector.tree.Position;

/**
 * Implementation of the add node edit operation.
 *
 * @author Martin Svoboda
 */
public class EditAddNode extends Edit {

    // -------------------------------------------------------------------------


    /**
     * Reference to the associated repairing instruction.
     */
    private InstructionAddNode repairingInstr;


    // -------------------------------------------------------------------------


    /**
     * Creates a new add node edit operation object.
     * @param repairingInstr Underlying repairing instruction.
     * @param basePosition Operation base position.
     */
    public EditAddNode(
        InstructionAddNode repairingInstr,
        Position basePosition
    ) {
        super(basePosition);
        this.repairingInstr = repairingInstr;
    }


    // -------------------------------------------------------------------------


    public Instruction getRepairingInstruction() {
        return this.repairingInstr;
    }

    /**
     * Returns the typed reference to the associated instruction.
     * @return Underlying repairing instruction reference.
     */
    public InstructionAddNode getAddLeafInstruction() {
        return this.repairingInstr;
    }

    public String getOperationTitle() {
        return "addNode(" +
            this.basePosition.getPositionString() + ", " +
            this.repairingInstr.getSiblingsCount() + ", " +
            this.repairingInstr.getNewLabel() + ")";
    }


    // -------------------------------------------------------------------------

    
    public EditAddNode cloneIdenticalOperation() {
        return new EditAddNode(
            this.repairingInstr,
            basePosition.cloneIdenticalPosition()
        );
    }
    
    
    public EditAddNode cloneShiftedOperation(int incrementValue) {
        return new EditAddNode(
            this.repairingInstr,
            basePosition.cloneShiftedPosition(incrementValue)
        );
    }


    // -------------------------------------------------------------------------

}
