
package cz.cuni.mff.corrector.edit;

import cz.cuni.mff.corrector.instruction.Instruction;
import cz.cuni.mff.corrector.instruction.InstructionRemoveLeaf;
import cz.cuni.mff.corrector.tree.Position;

/**
 * Implementation of the remove leaf edit operation.
 *
 * @author Martin Svoboda
 */
public class EditRemoveLeaf extends Edit {

    // -------------------------------------------------------------------------


    /**
     * Reference to the associated repairing instruction.
     */
    private InstructionRemoveLeaf repairingInstr;


    // -------------------------------------------------------------------------


    /**
     * Creates a new remove leaf edit operation object.
     * @param repairingInstr Underlying repairing instruction.
     * @param basePosition Operation base position.
     */
    public EditRemoveLeaf(
        InstructionRemoveLeaf repairingInstr,
        Position basePosition
    ) {
        super(basePosition);
        this.repairingInstr = repairingInstr;
    }


    // -------------------------------------------------------------------------


    public Instruction getRepairingInstruction() {
        return this.repairingInstr;
    }

    /**
     * Returns the typed reference to the associated instruction.
     * @return Underlying repairing instruction reference.
     */
    public InstructionRemoveLeaf getRemoveLeafInstruction() {
        return this.repairingInstr;
    }

    public String getOperationTitle() {
        return "removeLeaf(" + this.basePosition.getPositionString() + ")";
    }


    // -------------------------------------------------------------------------


    public EditRemoveLeaf cloneIdenticalOperation() {
        return new EditRemoveLeaf(
            this.repairingInstr,
            basePosition.cloneIdenticalPosition()
        );
    }
    
    
    public EditRemoveLeaf cloneShiftedOperation(int incrementValue) {
        return new EditRemoveLeaf(
            this.repairingInstr,
            basePosition.cloneShiftedPosition(incrementValue)
        );
    }


    // -------------------------------------------------------------------------

}
