
package cz.cuni.mff.corrector.edit;

import cz.cuni.mff.corrector.instruction.Instruction;
import cz.cuni.mff.corrector.instruction.InstructionRemoveNode;
import cz.cuni.mff.corrector.tree.Position;

/**
 * Implementation of the remove node edit operation.
 *
 * @author Martin Svoboda
 */
public class EditRemoveNode extends Edit {

    // -------------------------------------------------------------------------


    /**
     * Reference to the associated repairing instruction.
     */
    private InstructionRemoveNode repairingInstr;


    // -------------------------------------------------------------------------


    /**
     * Creates a new remove node edit operation object.
     * @param repairingInstr Underlying repairing instruction.
     * @param basePosition Operation base position.
     */
    public EditRemoveNode(
        InstructionRemoveNode repairingInstr,
        Position basePosition
    ) {
        super(basePosition);
        this.repairingInstr = repairingInstr;
    }


    // -------------------------------------------------------------------------


    public Instruction getRepairingInstruction() {
        return this.repairingInstr;
    }

    /**
     * Returns the typed reference to the associated instruction.
     * @return Underlying repairing instruction reference.
     */
    public InstructionRemoveNode getRemoveNodeInstruction() {
        return this.repairingInstr;
    }

    public String getOperationTitle() {
        return "removeNode(" + this.basePosition.getPositionString() + ")";
    }


    // -------------------------------------------------------------------------


    public EditRemoveNode cloneIdenticalOperation() {
        return new EditRemoveNode(
            this.repairingInstr,
            basePosition.cloneIdenticalPosition()
        );
    }
    
    
    public EditRemoveNode cloneShiftedOperation(int incrementValue) {
        return new EditRemoveNode(
            this.repairingInstr,
            basePosition.cloneShiftedPosition(incrementValue)
        );
    }


    // -------------------------------------------------------------------------

}
