
package cz.cuni.mff.corrector.execution;

import cz.cuni.mff.corrector.graph.Edge;
import cz.cuni.mff.corrector.graph.Vertex;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * Data report about a selected (promising) reached vertex.
 * Only suitable for the REFINE algorithm strategy.
 * 
 * @author Martin Svoboda
 */
public class Record {
    
    // -------------------------------------------------------------------------
    
    
    /**
     * Reference to the associated vertex object.
     */
    public Vertex associatedVertex;
    
    
    // -------------------------------------------------------------------------
    
    
    /**
     * The minimal cost over all open ingoing edges or <code>null</code>.
     */
    public Integer openMinimum;
    
    /**
     * The minimal cost over all closed ingoing edges or <code>null</code>.
     */
    public Integer closedMinimum;
    
    
    /**
     * The minimal cost over all ingoing edges or <code>null</code>.
     */
    public Integer generalMinimum;
    
    
    // -------------------------------------------------------------------------
    
    
    /**
     * The second minimal cost over all open ingoing edges if any.
     * This item serves as a base for assigning granted execution quota.
     * Only relevant with respect to processing of the incomplete vertex!
     */
    public Integer quotaBoundary;
    
    
    /**
     * Prepared set of open edges having the minimal cost over them.
     * Only relevant with respect to processing of the incomplete vertex!
     */
    public List<Edge> promisingEdges;
    
    
    // -------------------------------------------------------------------------
    
    /**
     * Final tracing cost to the associated target vertex.
     * Only relevant with respect to processing of the complete vertex!
     */
    public Integer tracingCost;
    
    /**
     * Prepared set of predecessor edges on shortest paths.
     * Only relevant with respect to processing of the complete vertex!
     */
    public Set<Edge> tracingEdges;
    
    
    // -------------------------------------------------------------------------
    
    
    /**
     * Prepared set of edges to be delayed.
     * Only relevant with respect to processing of the complete vertex!
     */
    public Set<Edge> delayedEdges;
    
    /**
     * Prepared set of edges that are closed or open but not to be delayed.
     * In other words this set contains all edges which locally cached
     * task (if any) is not required to be kept any longer.
     * Only relevant with respect to processing of the complete vertex!
     */
    public Set<Edge> uselessEdges;
    
    
    // -------------------------------------------------------------------------
    
    
    /**
     * Creates a new and empty instance of the report.
     * All internal components are prepared and ready to be used.
     * @param associatedVertex Vertex to be associated with the report.
     */
    public Record(Vertex associatedVertex) {
        
        this.associatedVertex = associatedVertex;
        
        this.openMinimum = null;
        this.closedMinimum = null;
        this.generalMinimum = null;
        
        this.quotaBoundary = null;
        this.promisingEdges = new ArrayList<Edge>();
        
        this.tracingCost = null;
        this.tracingEdges = new HashSet<Edge>();
        
        this.delayedEdges = new HashSet<Edge>();
        this.uselessEdges = new HashSet<Edge>();
        
    }
    
    
    // -------------------------------------------------------------------------
    
}
