
package cz.cuni.mff.corrector.execution;

import cz.cuni.mff.corrector.graph.Edge;
import java.util.ArrayList;
import java.util.List;

/**
 * Data container for maintaining scheduling requests.
 * This class is only relevant for scheduling approaches.
 * 
 * Also note that this class only encapsulates all the required containers,
 * but no functionality is provided, so they can be used directly by users.
 * 
 * @author Martin Svoboda
 */
public class Requests {
    
    // -------------------------------------------------------------------------
    
    
    /**
     * List of requested nested tasks during the last execution.
     * When a new request is created, the given task is inserted into this
     * internal list. When the execution of the parent task is revisited
     * once again, these requested tasks are closed, i.e. the corresponding
     * edges are bound with the nested repair structure.
     * Note that the caller is responsible to maintain consistency between
     * this tasks container and the analogous container for edges!
     */
    public List<Task> requestedTasks;
    
    /**
     * List of requested nested edges during the last execution.
     * Note that the caller is responsible to maintain consistency between
     * this edges container and the analogous container for tasks!
     * It means that they contain the corresponding pairs of edges and
     * tasks and that the order of items is mutually the same.
     */
    public List<Edge> requestedEdges;
    
    
    // -------------------------------------------------------------------------
    
    
    /**
     * List of postponed and not yet evaluated edges and their intents.
     * Only relevant for the following configurations:
     *   DEFAULT strategy: SCHEDULING_SINGLE approach,
     *   EXPLORING strategy: SCHEDULING_SINGLE approach.
     * When a new reached vertex is processed, only one request is allowed,
     * so the remaining prepared nested intents are stored in this list
     * and processed later on when the handler is revisited once again.
     */
    public List<Edge> preparedEdges;
    
    
    // -------------------------------------------------------------------------
    
    
    /**
     * List of postponed and not yet evaluated tasks and their intents.
     * Only relevant for the following configurations:
     *   ITERATIVE strategy: SCHEDULING_SINGLE approach.
     */
    public List<Task> preparedTasks;
    
    /**
     * List of postponed and not yet evaluated tasks and their intents.
     * Only relevant for the following configurations:
     *   ITERATIVE strategy: SCHEDULING_SINGLE approach.
     */
    public List<Quota> preparedQuotas;
    
    
    // -------------------------------------------------------------------------
    
    
    /**
     * Creates a new instance of the requests container.
     */
    public Requests() {
        this.requestedTasks = new ArrayList<Task>();
        this.requestedEdges = new ArrayList<Edge>();
        this.preparedEdges = new ArrayList<Edge>();
        this.preparedTasks = new ArrayList<Task>();
        this.preparedQuotas = new ArrayList<Quota>();
    }
    
    
    // -------------------------------------------------------------------------
    
}
