
package cz.cuni.mff.corrector.expression;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * Regular subexpression for Kleene star operation.
 *
 * @author Martin Svoboda
 */
public class ExpressionStar extends Expression {

    // -------------------------------------------------------------------------


    /**
     * Operand subexpression of the Kleene star operation.
     */
    private Expression leftOperand;


    // -------------------------------------------------------------------------


    /**
     * Creates an instance of the Kleene star regular subexpression.
     * @param leftOperand Left subexpression of the concatenation operation.
     */
    protected ExpressionStar(Expression leftOperand) {
        this.leftOperand = leftOperand;
    }


    public ExpressionStar duplicateExpression() {
        return new ExpressionStar(this.leftOperand.duplicateExpression());
    }


    // -------------------------------------------------------------------------


    public Type getType() {
        return Type.STAR;
    }


    public String serializeExpression() {
        return this.leftOperand.serializeExpression() + TOKEN_OP_STAR;
    }


    public Set<String> getSymbols() {
        Set<String> symbols = new HashSet<String>(leftOperand.getSymbols());
        return symbols;
    }


    protected void markSymbols(Provider provider) {
        this.leftOperand.markSymbols(provider);
    }


    // -------------------------------------------------------------------------


    public boolean resolveEmpty() {
        this.allowsEmpty = true;
        return this.allowsEmpty;
    }

    public Set<String> resolveFirst() {
        Set<String> symbols = new HashSet<String>(leftOperand.resolveFirst());
        return symbols;
    }

    public Set<String> resolveLast() {
        Set<String> symbols = new HashSet<String>(leftOperand.resolveLast());
        return symbols;
    }

    public Map<String, Set<String>> resolveFollow() {
        Map<String, Set<String>> mapping = new HashMap<String, Set<String>>();
        for (String keySymbol : leftOperand.resolveLast()) {
            Set<String> symbols =
                new HashSet<String>(leftOperand.resolveFirst());
            mapping.put(keySymbol, symbols);
        }
        Map<String, Set<String>> leftMap = leftOperand.resolveFollow();
        for (String keySymbol : leftMap.keySet()) {
            if (mapping.containsKey(keySymbol)) {
                mapping.get(keySymbol).addAll(leftMap.get(keySymbol));
            } else {
                Set<String> newSymbols =
                    new HashSet<String>(leftMap.get(keySymbol));
                mapping.put(keySymbol, newSymbols);
            }
        }
        return mapping;
    }


    // -------------------------------------------------------------------------

}
