
package cz.cuni.mff.corrector.instruction;

import cz.cuni.mff.corrector.edit.Edit;

/**
 * Ancestor for repairing instructions.
 *
 * @author Martin Svoboda
 */
public abstract class Instruction {

    // -------------------------------------------------------------------------


    /**
     * Enumeration of all repairing instructions types.
     */
    public enum Type {
        /**
         * Instruction type: add leaf.
         */
        ADD_LEAF,
        /**
         * Instruction type: remove leaf.
         */
        REMOVE_LEAF,
        /**
         * Instruction type: rename label.
         */
        RENAME_LABEL,
        /**
         * Instruction type: add node.
         */
        ADD_NODE,
        /**
         * Instruction type: remove node.
         */
        REMOVE_NODE;
    }


    // -------------------------------------------------------------------------


    /**
     * Default unit cost for edit operations and thus repairing instructions.
     */
    public static int COST_UNIT = 1;

    /**
     * Undefined or zero cost named constant.
     */
    public static int COST_ZERO = 0;


    // -------------------------------------------------------------------------


    /**
     * Returns the type of the repairing instruction.
     * @return Instruction type enumeration constant.
     */
    public abstract Type getInstructionType();


    /**
     * Returns the cost of the particular repairing instruction.
     * @return Repairing instruction cost.
     */
    public abstract int getInstructionCost();


    /**
     * Returns the readable title of the given repairing instruction.
     * @return Repairing instruction title.
     */
    public abstract String getTitleString();


    // -------------------------------------------------------------------------


    /**
     * Translates the given repairng instruction into the edit operation.
     * @return Corresponding edit operation.
     */
    public abstract Edit generateEditOperation();


    // -------------------------------------------------------------------------

}
