
package cz.cuni.mff.corrector.instruction;

import cz.cuni.mff.corrector.edit.EditAddNode;
import cz.cuni.mff.corrector.tree.Position;

/**
 * Repairing instruction add node.
 *
 * @author Martin Svoboda
 */
public class InstructionAddNode extends Instruction {

    // -------------------------------------------------------------------------


    /**
     * New label associated with the add node instruction.
     */
    private final String newLabel;

    /**
     * Number of sibling nodes to be pushed lower.
     */
    private final int siblingsCount;


    // -------------------------------------------------------------------------


    /**
     * Creates a new instance of an add node instruction.
     * @param newLabel New label value.
     * @param siblingsCount Number of sibling nodes to be pushed lower.
     */
    public InstructionAddNode(String newLabel, int siblingsCount) {
        this.newLabel = newLabel;
        this.siblingsCount = siblingsCount;
    }


    // -------------------------------------------------------------------------


    /**
     * Returns the associated new label.
     * @return New label associated with the instruction.
     */
    public String getNewLabel() {
        return this.newLabel;
    }

    /**
     * Returns the associated number of sibling nodes.
     * @return Number of sibling nodes to be pushed down.
     */
    public int getSiblingsCount() {
        return this.siblingsCount;
    }


    // -------------------------------------------------------------------------


    public Type getInstructionType() {
        return Type.ADD_NODE;
    }

    public int getInstructionCost() {
        return COST_UNIT;
    }

    public String getTitleString() {
        return "addNode(" + this.siblingsCount + ", " + this.newLabel + ")";
    }
    

    // -------------------------------------------------------------------------


    public EditAddNode generateEditOperation() {
        return new EditAddNode(this, new Position(0));
    }


    // -------------------------------------------------------------------------

}
