
package cz.cuni.mff.corrector.repair;

import cz.cuni.mff.corrector.edit.Edit;
import java.util.List;

/**
 * Mend encapsulation for edit sequences repairing sequences and intents.
 *
 * @author Martin Svoboda
 */
public class Mend {

    // -------------------------------------------------------------------------


    /**
     * Particular sequence of edit operations.
     */
    private List<Edit> editSequence;


    /**
     * Size of the positions shifting on the top level.
     */
    private int positionShift;


    // -------------------------------------------------------------------------


    /**
     * Creates a new mend instance.
     * @param editSequence Reference to the particular edit sequence.
     * @param positionShift Positions shifting size.
     */
    public Mend(List<Edit> editSequence, int positionShift) {
        this.editSequence = editSequence;
        this.positionShift = positionShift;
    }


    // -------------------------------------------------------------------------


    /**
     * Returns the reference to the internal edit sequence container.
     * @return Particular sequence of edit operations.
     */
    public List<Edit> getEditSequence() {
        return this.editSequence;
    }
    
    
    /**
     * Returns the size of positions shifting.
     * @return Size of the positions shifting on the top level.
     */
    public int getShiftingSize() {
        return this.positionShift;
    }
    
    
    // -------------------------------------------------------------------------


    /**
     * Applies modPre to all edit operations stored in the mend.
     * @param newPrefix Simple position to be prepended.
     */
    public void applyRootPrepending(Integer newPrefix) {
        for (Edit edit : this.editSequence) {
            edit.applyRootPrepending(newPrefix);
        }
    }


    /**
     * Applies modCut to all edit operations stored in the mend.
     */
    public void applyRootCutting() {
        for (Edit edit : this.editSequence) {
            edit.applyRootCutting();
        }
    }


    // -------------------------------------------------------------------------


    /**
     * Returns the serialized mend.
     * @param prefixString String added before each operation.
     * @param suffixString String added after each operation.
     * @return Sequence of lines with particular edit operations.
     */
    public String getMendSerialization(
        String prefixString,
        String suffixString
    ) {

        StringBuilder builder = new StringBuilder();

        for (Edit operation : editSequence) {
            builder.append(prefixString);
            builder.append(operation.getOperationTitle());
            builder.append(suffixString);
        }

        return builder.toString();

    }


    // -------------------------------------------------------------------------

}
