
package cz.cuni.mff.corrector.testing;

/**
 * Simple timer implementation for measuring duration of executions.
 *
 * @author Martin Svoboda
 */
public class Timer {
    
    // -------------------------------------------------------------------------
    
    
    /**
     * The overall time period between the timer start and stop.
     * Note that the elapsed time is measured in nanoseconds.
     */
    private Long measuredTime;
    
    
    /**
     * Timestamp of the very moment when the timer was started.
     */
    private Long stampStarted;
    
    /**
     * Timestamp of the very moment when the timer was stopped.
     */
    private Long stampStopped;


    // -------------------------------------------------------------------------
    
    
    /**
     * Creates a new one time profiling timer instance.
     */
    public Timer() {
        this.measuredTime = null;
        this.stampStarted = null;
        this.stampStopped = null;
    }
    
    
    // -------------------------------------------------------------------------
    
    
    /**
     * Starts the timer, i.e. records the current timestamp as the starting one.
     */
    public void startProfilingTimer() {
        this.stampStarted = System.nanoTime();
    }


    /**
     * Starts the timer, i.e. records the current timestamp as the starting one.
     * Also the overall measured period is computed by this function.
     */
    public void stopProfilingTimer() {
        this.stampStopped = System.nanoTime();
        this.measuredTime = this.stampStopped - this.stampStarted;
    }
    
    
    /**
     * Returns the overall elapsed time measured in nanoseconds.
     * @return Period length in nanoseconds.
     */
    public long getElapsedNanos() {
        return this.measuredTime;
    }
    
    /**
     * Returns the overall elapsed time measured in miliseconds.
     * @return Period length in miliseconds.
     */
    public long getElapsedMilis() {
        return this.measuredTime / 1000000;
    }
    
    
    // -------------------------------------------------------------------------
    
}
