
package cz.cuni.mff.corrector.grammar;

import java.util.Arrays;
import java.util.Set;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 * Testing class for regular tree grammars.
 *
 * @author Martin Svoboda
 */
public class GrammarTest {

    // -------------------------------------------------------------------------


    /**
     * Tests the correctness of grammar object creation.
     */
    @Test
    public void testGrammarCreation() {

        // Definition of testing grammars
        // - list of starting nonterminals and lines with production rules
        // - array of starting nonterminals
        // - array of nonterminal symbols
        // - array of terminal symbols
        String[][][] container = {
            {
                {"root\n" + "root root a|b|c\n" + "a a #empty\n" +
                    "b b #empty\n" + "c c a*.b*\n"},
                {"root"},
                {"root", "a", "b", "c"},
                {"root", "a", "b", "c"}
            },
            {
                {"ROOT\n" + "ROOT root A|B|C\n" + "A a #empty\n" +
                    "B b #empty\n" + "C c A*.B*\n"},
                {"ROOT"},
                {"ROOT", "A", "B", "C"},
                {"root", "a", "b", "c"}
            },
        };

        // Perfrorming grammar creation test
        for (String[][] input : container) {
            Grammar grammar = Grammar.createFromDataString(input[0][0]);
            assertNotNull(grammar);
            // Printing grammar debug dump
            System.out.println("GRAMMAR");
            grammar.printDebugDump("");
            // Testing starting symbols
            Set<String> starting = grammar.getStartingSymbols();
            assertTrue(starting.containsAll(Arrays.asList(input[1])));
            assertEquals(input[1].length, starting.size());
            // Testing nonterminal symbols
            Set<String> nonterminals = grammar.getNonterminalSymbols();
            assertTrue(nonterminals.containsAll(Arrays.asList(input[2])));
            assertEquals(input[2].length, nonterminals.size());
            // Testing terminal symbols
            Set<String> terminals = grammar.getTerminalSymbols();
            assertTrue(terminals.containsAll(Arrays.asList(input[3])));
            assertEquals(input[3].length, terminals.size());
        }

    }


    // -------------------------------------------------------------------------
    
}
