
package cz.cuni.mff.corrector.tree;

import java.util.ArrayList;
import java.util.List;
import junit.framework.Assert;
import org.junit.Test;

/**
 * Basic tests of operations over positions.
 * 
 * @author Martin Svoboda
 */
public class PositionTest {
    
    // -------------------------------------------------------------------------
    
    
    /**
     * Tests construction of position objects.
     */
    @Test
    public void testPositionCreation() {
        
        Position p;
        
        p = new Position();
        Assert.assertEquals("e", p.getPositionString());
        
        p = new Position(5);
        Assert.assertEquals("5", p.getPositionString());
        
        List<Integer> v = new ArrayList<Integer>();
        p = new Position(v);
        Assert.assertEquals("e", p.getPositionString());
        
        v.add(1);
        v.add(2);
        v.add(3);
        p = new Position(v);
        Assert.assertEquals("1.2.3", p.getPositionString());
        
    }
    
    
    /**
     * Tests basic operations for modifying positions.
     */
    @Test
    public void testPositionOperations() {
        
        Position p = new Position();
        
        p.applyRootPrepending(1);
        Assert.assertEquals("1", p.getPositionString());
        
        p.applyRootPrepending(2);
        Assert.assertEquals("2.1", p.getPositionString());
        
        p.applyRootCutting();
        Assert.assertEquals("1", p.getPositionString());
        
        p.applyRootShifting(5);
        Assert.assertEquals("6", p.getPositionString());
        
        p.applyRootPrepending(3);
        Assert.assertEquals("3.6", p.getPositionString());
        
        p.applyRootPrepending(7);
        Assert.assertEquals("7.3.6", p.getPositionString());
        
        p.applyRootShifting(0);
        Assert.assertEquals("7.3.6", p.getPositionString());
        
        p.applyRootCutting();
        Assert.assertEquals("3.6", p.getPositionString());
        
        p.applyRootCutting();
        Assert.assertEquals("6", p.getPositionString());
        
        p.applyRootCutting();
        Assert.assertEquals("e", p.getPositionString());
        
    }
    
    
    /**
     * Tests independent copying functions.
     */
    @Test
    public void testCopyingOperations() {
        
        Position s, t;
        List<Integer> v = new ArrayList<Integer>();

        v.add(1);
        v.add(2);
        s = new Position(v);
        t = s.cloneShiftedPosition(5);
        Assert.assertEquals("6.2", t.getPositionString());
        Assert.assertNotSame(s, t);
        
    }
    
    
    // -------------------------------------------------------------------------
    
}
